/*
 *
 * Copyright (c) 2003 by FORCE Computers.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  This
 * file and program are licensed under a BSD style license.  See
 * the Copying file included with the OpenHPI distribution for
 * full licensing terms.
 *
 * Authors:
 *     Thomas Kanngieser <thomas.kanngieser@fci.com>
 */

#include <stdio.h>
#include <stdlib.h>

#include "ipmi.h"


static GHashTable *AllocParams();
static gboolean RemoveParam( gpointer key, gpointer value, gpointer  user_data );
static void DestryParams( GHashTable *params );


static const char *plugin_params[] =
{
  "entity_root", "{SYSTEM_CHASSIS,2}",

#if 1
  "name",       "lan",

#if 0
  "addr",       "192.168.110.187",
  "port",       "623",
#else
  "addr",       "localhost",
  "port",       "4711",
#endif

  "auth_type",  "md2",
  "auth_level", "admin",
  "username",   "kanne",
  "password",   "kanne",
#else
  "name",       "file",
  "file",       "ipmi.con",
#endif
  "logfile",    "log",
  "logfile_max", "10",
  0
};

static GHashTable *
AllocParams()
{
  GHashTable *params = g_hash_table_new( g_str_hash, g_str_equal );

  const char **p = plugin_params;

  while( *p )
     {
       const char *pp = *p;
       
       char *v0 = new char[strlen( pp ) + 1];
       strcpy( v0, pp );

       pp = *(p+1);
       char *v1 = new char[strlen( pp ) + 1];
       strcpy( v1, pp );

       g_hash_table_insert( params, v0, v1 );

       p++;
       p++;
     }

  return params;
}


static gboolean 
RemoveParam( gpointer  /*key*/,
             gpointer  /*value*/,
             gpointer  /*user_data*/ )
{
  return 1;
}


static void
DestryParams( GHashTable *params )
{
  g_hash_table_foreach_remove( params, RemoveParam, 0 );
  g_hash_table_destroy( params );
}

/*
static void
TestIpmi( tIpmiCon *con )
{
  tIpmiMsg                 msg;
  tIpmiMsg                 rsp_msg;
  int			   rv;
  tIpmiSystemInterfaceAddr addr;
  unsigned int manufacturer_id;
  unsigned int product_id;

  memset( &addr, 0, sizeof( tIpmiAddr ) );
  
  addr.addr_type = dIpmiSystemInterfaceAddrType;
  addr.channel   = 0xf;
  addr.lun       = 0;

  msg.netfn    = dIpmiAppNetfn;
  msg.cmd      = dIpmiGetDeviceIdCmd;
  msg.data_len = 0;

  rv = IpmiConExecuteCmd( con, (tIpmiAddr *)&addr, sizeof( tIpmiSystemInterfaceAddr ),
                          &msg, &rsp_msg );

  if ( rv )
     {
       printf( "IpmiConExecuteCmd: %d, %s\n", rv, strerror( rv ) );
       return;
     }

  manufacturer_id =    (rsp_msg.data[7]
                        | (rsp_msg.data[8] << 8)
                        | (rsp_msg.data[9] << 16));
  product_id      = rsp_msg.data[10] | (rsp_msg.data[11] << 8);

  printf( "manufacturer = 0x%04x\nproduct = 0x%04x\n",
          manufacturer_id, product_id );
}
*/


static void
TestInterface( struct oh_abi_v2 *abi, void *hdl )
{
  abi->discover_resources( hdl );

  while( 1 )
     {
       struct oh_event event;
       struct timeval to = { 0, 0 };

       int rv = abi->get_event( hdl, &event, &to );

       if ( rv )
            printf( "********** read event *********\n" );
       
       usleep( 100000 );
     }
}


__BEGIN_DECLS
int ipmidirect_get_interface(void **pp, const uuid_t uuid);
__END_DECLS


int
main( int /*argc*/, char * /*argv*/ [] )
{
  void *dummy;
  struct oh_abi_v2 *abi = 0;
  GHashTable *params;
  void *hdl;
  uuid_t myuid;

  oh_uid_initialize();

  memcpy( &myuid, UUID_OH_ABI_V2, sizeof(myuid) );

  if (    ipmidirect_get_interface( &dummy, myuid ) < 0
       || dummy == 0 )
     {
       printf( "cannot get interface !\n" );
       return 10;
     }

  abi = (struct oh_abi_v2 *) dummy;

  if ( abi->open == 0 )
     {
       printf( "cannot get interface !\n" );
       return 10;
     }
	  
  params = AllocParams();

  hdl = abi->open( params );

  if ( hdl == 0 )
     {
       printf( "cannot open plugin !\n" );
       return 10;
     }

  printf( "ready.\n" );
  TestInterface( abi, hdl );

  abi->close( hdl );
  DestryParams( params );

  return 0;
}
